from django.template.defaultfilters import slugify


class AutoStrMixin(object):
    """ Allows string fields ``__autostr__``, ``__autounicode__`` and
        ``__autorepr__`` to be used to implement ``__str__``, ``__unicode__``
        and ``__repr__``.  Additionally, if no ``__str__`` or ``__autostr__``
        has been implemented, ``__str__`` will return
        ``__unicode__().encode("utf-8")``.

        WARNING: This has some problems and probably shouldn't be used.
        Instead, see ``autorepr``, ``autounicode``, and autostr, below.

        ::
            >>> class Foo(AutoStrMixin):
            ...     def __init__(self, name):
            ...         self.name = name
            ...
            ...     __autounicode__ = u"name: {self.name}"
            ...     __autorepr__ = "name={self.name!r}"
            ...
            >>> foo = Foo(u"wolever")
            >>> unicode(foo)
            u'name: wolever'
            >>> str(foo)
            'name: wolever'
            >>> repr(foo)
            "<__main__.Foo name=u'wolever' at 0x...>"
        """

    def _autostr_helper(self, method, default):
        formatstr = getattr(self, "__auto%s__" %(method, ), None)
        if formatstr is None:
            return default()
        return formatstr.format(self=self)

    def __unicode__(self):
        default = lambda: unicode(repr(self))
        return self._autostr_helper("unicode", default)

    def __str__(self):
        default = lambda: unicode(self).encode("utf-8")
        return self._autostr_helper("str", default)

    def __repr__(self):
        result = self._autostr_helper("repr", lambda: None)
        if result is None:
            return super(AutoStrMixin, self).__repr__()
        return "<%s.%s %s at 0x%x>" %(self.__class__.__module__,
                                      self.__class__.__name__,
                                      result, id(self))


def _autofmthelper(name, fmt, postprocess=None):
    def fmtfunc(self):
        result = fmt.format(self=self)
        if postprocess is not None:
            result = postprocess(self, result)
        return result
    fmtfunc.__name__ = name
    return fmtfunc

def autounicode(fmt):
    """ Returns a simple ``__unicode__`` function::

        >>> class Person(object):
        ...     name = "Alex"
        ...     __unicode__ = autounicode("{self.name}")
        ...
        >>> unicode(Person())
        u'Alex'
        >>>
        """
    return _autofmthelper("__unicode__", unicode(fmt))

def autostr(fmt):
    """ Returns a simple ``__str__`` function::

        >>> class Person(object):
        ...     name = "Alex"
        ...     __str__ = autostr("{self.name}")
        ...
        >>> str(Person())
        'Alex'
        >>>
        """
    fmt = str(fmt)
    return _autofmthelper("__str__", str(fmt))

def autorepr(fmt):
    """ Returns a simple ``__repr__`` function::

        >>> class Person(object):
        ...     name = "Alex"
        ...     __repr__ = autorepr("name={self.name!r}")
        ...
        >>> repr(Person())
        "<__main__.Person name='Alex' at 0x...>"
        >>>
        """
    return _autofmthelper("__repr__", str(fmt), lambda self, result: (
        "<%s.%s %s at 0x%x>" %(
            self.__class__.__module__,
            self.__class__.__name__,
            result,
            id(self),
        )
    ))


def slug_property(field):
    @property
    def slug_property_helper(self):
        return slugify(getattr(self, field))
    return slug_property_helper

if __name__ == "__main__":
    import doctest
    doctest.testmod(optionflags=doctest.ELLIPSIS)



import sys

from ..discover_providers import discover_providers

# Note: these tests will be run in the context of "helper_project/"

def test_discover_providers():
    """Test that discover_providers will actually discover and import a
       module."""

    def find_helper_module():
        for mod in sys.modules.iterkeys():
            if "example_provider" in mod:
                return mod
        return None

    # Make sure that the 'example_provider' module isn"t already in
    # sys.modules
    if find_helper_module():
        del sys.modules[find_helper_module()]

    discover_providers("example_provider")

    # Make sure the module has been re-added to sys.modules (that is, it has
    # been imported)
    assert find_helper_module()



import os
import sys
import logging
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.importlib import import_module
from django.core.management import call_command
from django.core.management.commands.dumpdata import Command as DumpData

log = logging.getLogger("dump_app_data")

HELP = """\
Dumps data for each app to the directory returned by
``settings.DUMP_APP_DATA_FUNC``. If none is specified, a default function will
be used which dumps data to ``$APP/fixtures/${FIXTURE-dev}.${FORMAT-json}`` for
all apps which share a prefix with ``ROOT_URLCONF``.
"""


class Command(BaseCommand):
    help = HELP
    can_import_settings = True
    option_list = DumpData.option_list + (
        make_option('--fixture-name', dest='fixture_name', default='dev',
                    help='Fixture name to write to'),
    )
    
    @classmethod
    def app_name_to_path(cls, app_name):
        app_module = import_module(app_name)
        return os.path.join(os.path.dirname(app_module.__file__))

    @classmethod
    def DEFAULT_DUMP_APP_DATA_FUNC(cls, options, app_name):
        prefix = settings.ROOT_URLCONF.split('.')[0]
        if not app_name.startswith(prefix):
            log.debug("app %r is not prefixed with %r; ignoring",
                      app_name, prefix)
            return None
        return os.path.join(cls.app_name_to_path(app_name), "fixtures",
                            options["fixture_name"] + "." + options["format"])

    @classmethod
    def dump_app_data(cls, options, app_name, output_file):
        dumpdata_options = dict(options)
        dumpdata_options.pop("fixture_name")

        output_path = os.path.dirname(output_file)
        if not os.path.exists(output_path):
            os.makedirs(output_path)

        orig_stdout = sys.stdout
        try:
            sys.stdout = open(output_file + ".tmp", "w")
            call_command("dumpdata", app_name.split(".")[-1],
                         **dumpdata_options)
        finally:
            sys.stdout.close()
            sys.stdout = orig_stdout

        os.rename(output_file + ".tmp", output_file)
    
        try:
            output = open(output_file, "r")
            if len(output.read(16)) < 10:
                os.remove(output_file)
                print "no data for %r; removing %r" %(app_name, output_file)
            else:
                print "dumped data for %r to %r" %(app_name, output_file)
        finally:
            output.close()

    def handle(self, *args, **options):
        path_func = getattr(settings, "DUMP_APP_DATA_FUNC",
                            self.DEFAULT_DUMP_APP_DATA_FUNC)
        for app_name in settings.INSTALLED_APPS:
            path = path_func(options, app_name)
            if not path:
                continue
            self.dump_app_data(options, app_name, path)



import re

from markupsafe import Markup
from django import template

register = template.Library()

BR = Markup("<br />")
AHREF = Markup("<a href='%s' rel='nofollow'>%s</a>")

LINK_RE = re.compile(r"(http(s?)\://\S+)")
def link_re_sub(match):
    link = match.group(0)
    return AHREF %(link, link)

EMAIL_RE = re.compile(r"(\S{2,}@\S{2,}\.[a-zA-Z]{2,3})\.?")
def email_re_sub(match):
    email = match.group(0)
    return AHREF %("mailto:" + email, email)


@register.filter
def txt2html(text):
    text = text.strip()
    text = LINK_RE.sub(link_re_sub, text)
    text = EMAIL_RE.sub(email_re_sub, text)
    text = text.replace("\n", BR)
    return text



import textwrap

def truncate(s, max_len=80):
    if len(s) > max_len:
        return s[:max_len - 3] + "..."
    return s

def dedent(s):
    """ Similar to ``textwrap.dedent``, but possible to use like this::

        >>> print dedent('''
        ...     Hello,
        ...     world!
        ... ''')
        Hello,
        world!
        >>>
    """
    return textwrap.dedent(s.lstrip("\n").rstrip())

def to_str(obj, encoding='utf-8', **encode_args):
    r"""
    Returns a ``str`` of ``obj``, encoding using ``encoding`` if necessary. For
    example::

        >>> some_str = "\xff"
        >>> some_unicode = u"\u1234"
        >>> some_exception = Exception(u'Error: ' + some_unicode)
        >>> to_str(some_str)
        '\xff'
        >>> to_str(some_unicode)
        '\xe1\x88\xb4'
        >>> to_str(some_exception)
        'Error: \xe1\x88\xb4'
        >>> to_str([u'\u1234', 42])
        "[u'\\u1234', 42]"

    See source code for detailed semantics.
    """
    # We coerce to unicode if '__unicode__' is available because there is no
    # way to specify encoding when calling ``str(obj)``, so, eg,
    # ``str(Exception(u'\u1234'))`` will explode.
    if isinstance(obj, unicode) or hasattr(obj, "__unicode__"):
        # Note: unicode(u'foo') is O(1) (by experimentation)
        return unicode(obj).encode(encoding, **encode_args)

    # Note: it's just as fast to do `if isinstance(obj, str): return obj` as it
    # is to simply return `str(obj)`.
    return str(obj)

def to_unicode(obj, encoding='utf-8', fallback='latin1', **decode_args):
    r"""
    Returns a ``unicode`` of ``obj``, decoding using ``encoding`` if necessary.
    If decoding fails, the ``fallback`` encoding (default ``latin1``) is used.
    
    For example::

        >>> to_unicode('\xe1\x88\xb4')
        u'\u1234'
        >>> to_unicode('\xff')
        u'\xff'
        >>> to_unicode(u'\u1234')
        u'\u1234'
        >>> to_unicode(Exception(u'\u1234'))
        u'\u1234'
        >>> to_unicode([u'\u1234', 42])
        u"[u'\\u1234', 42]"

    See source code for detailed semantics.
    """

    if isinstance(obj, unicode) or hasattr(obj, "__unicode__"):
        return unicode(obj)

    obj_str = str(obj)
    try:
        return unicode(obj_str, encoding, **decode_args)
    except UnicodeDecodeError:
        return unicode(obj_str, fallback, **decode_args)

def to_base(number, alphabet):
    if not isinstance(number, (int, long)):
        raise TypeError('number must be an integer')
    if number < 0:
        raise ValueError('number must be nonnegative')

    # Special case for zero
    if number == 0:
        return '0'

    in_base = []
    while number != 0:
        number, i = divmod(number, len(alphabet))
        in_base.append(alphabet[i])
    return "".join(reversed(in_base))

def to36(number):
    return to_base(number, '0123456789abcdefghijklmnopqrstuvwxyz')

def from36(str):
    return int(str, 36)


if __name__ == "__main__":
    import doctest
    doctest.testmod(optionflags=doctest.ELLIPSIS)



import urlparse

from django.http import HttpResponseRedirect

def guess_next(request, default=None):
    if "next" in request.GET:
        return request.GET["next"]
    if default is None:
        default = request.META.get("HTTP_REFERER")
    if not default:
        return "/"
    parsed = urlparse.urlsplit(default)
    next = urlparse.urlunsplit(('', '') + parsed[2:])
    return next

def redirect_next(request, default=None):
    next = guess_next(request, default)
    if not next.startswith("/"):
        next = "/" + next
    return HttpResponseRedirect(next)




import pytz
from datetime import datetime

from django.template import Template, Context
from nose.tools import assert_equal

from ensi_common.django.templatetags import localdt
from ensi_common.testing import parameterized

class TestLocalDateTime(object):
    test_datetime = datetime.utcfromtimestamp(1313945312.076319)
    second_datetime = datetime.utcfromtimestamp(1313950000.0)
    test_tz = pytz.timezone("Japan")

    def _render(self, template, context):
        template = Template(template)
        context = Context(context)
        return template.render(context)

    @parameterized([
        ["localdt", "Aug. 22, 2011, 1:48 a.m."],
        ["localdate", "Aug. 22, 2011"],
        ["localtime", "1:48 a.m."],
    ])
    def test_conversion(self, tag, expected):
        template = "{%% load localdt %%}{%% %s some_dt %%}" %(tag, )
        context = {
            "localtz": self.test_tz,
            "some_dt": self.test_datetime,
        }
        result = self._render(template, context)
        assert_equal(result, expected)

    @parameterized([
        ["{% localtime other_tz some_dt %}", "6:48 p.m."],
        ["{% localtime other_tz some_dt as foo %}a{{ foo }}b", "a6:48 p.m.b"],
        ["{% localtime some_dt as foo %}a{{ foo }}b", "a1:48 a.m.b"],
        ["{% localtimesince some_dt %}", "1 hour, 18 minutes"],
        ["{% localtimesince other_tz some_dt %}", "1 hour, 18 minutes"],
        ["{% localtimesince other_tz some_dt other_dt %}", "4 hours, 4 minutes"],
        ["{% localtimesince some_dt as foo %}a{{ foo }}b", "a1 hour, 18 minutesb"],
        ["{% localtimesince other_tz some_dt as foo %}a{{ foo }}b", "a1 hour, 18 minutesb"],
        ["{% localtimesince other_tz some_dt other_dt as foo %}a{{ foo }}b", "a4 hours, 4 minutesb"],
    ])
    def test_parsing(self, input, expected):
        oldutcnow = localdt.utcnow
        localdt.utcnow = lambda: self.second_datetime
        try:
            template = "{% load localdt %}" + input
            context = {
                "localtz": self.test_tz,
                "other_tz": pytz.timezone("Africa/Johannesburg"),
                "some_dt": self.test_datetime,
                "other_dt": datetime.utcfromtimestamp(1313960000.0)
            }
            result = self._render(template, context)
            assert_equal(result, expected)
        finally:
            localdt.utcnow = oldutcnow



from numbers import Number
from datetime import datetime

import pytz
from django import template
from django.template.defaultfilters import timesince
from django.utils import formats

def utcnow():
    # note: needed so we can override this during tests
    return datetime.utcnow()

class LocalBase(template.Node):
    def __init__(self, args):
        self.args = args

    @classmethod
    def usage(cls):
        return template.TemplateSyntaxError(
            cls.usage %(cls.tag_name, )
        )

    @classmethod
    def parse(cls, parser, token):
        args = token.split_contents()[1:]
        if len(args) < 1:
            raise cls.usage()
        return cls(args)

    def _parse(self):
        context = self.context
        in_args = list(self.args)

        if hasattr(context.get(in_args[0]), "localize"):
            tz = context[in_args.pop(0)]
        else:
            tz = context["localtz"]
        self.tz = tz

        # datetime
        self.datetime = context[in_args.pop(0)]

        # arg
        arg = None
        if in_args and in_args[0] != "as":
            arg = in_args.pop(0)
        self.arg = arg

        # output_var
        output_var = None
        if in_args:
            if len(in_args) != 2:
                raise self.usage()
            output_var = in_args[1]
        self.output_var = output_var

    def _localize_datetime(self, tz, value):
        if isinstance(value, Number):
            value = datetime.utcfromtimestamp(value)
        value_utc = value.replace(tzinfo=pytz.utc)
        return tz.normalize(value_utc.astimezone(tz))

    def _render(self):
        raise Exception("Override this!")

    def render(self, context):
        self.context = context
        self._parse()
        value = self._render()

        if self.output_var:
            context[self.output_var] = value
            return ""
        return value

class LocalDateTimeBase(LocalBase):
    """ Returns a localized datetime, date or time.

            {% local{dt,date,time} [tz] datetime [format] [as name] %}

        If ``tz`` is not specified the ``localtz`` context variable is used.

        Formatting is done by ``django.utils.formats``. """

    usage = "usage: {%% %s [tz] datetime [format] [as name] %%}"

    def _render(self):
        format = self.arg
        localized = self._localize_datetime(self.tz, self.datetime)
        return self.format_func(localized, format)


class LocalDateTime(LocalDateTimeBase):
    tag_name = "localdt"

    def format_func(self, value, format=None):
        format = format or "DATETIME_FORMAT"
        return formats.date_format(value, format)


class LocalDate(LocalDateTimeBase):
    tag_name = "localdate"
    format_func = staticmethod(formats.date_format)


class LocalTime(LocalDateTimeBase):
    tag_name = "localtime"
    format_func = staticmethod(formats.time_format)


class LocalTimesince(LocalBase):
    """ Returns a localized timesince.

            {% timesince [tz] datetime [target] [as name] %} """

    tag_name = "localtimesince"
    usage = "usage: {%% %s [tz] datetime [target] [as name] %%}"

    def _render(self):
        target = self.arg
        if target is None:
            target = utcnow()
        else:
            target = self.context[target]
        target = self._localize_datetime(self.tz, target)
        localized = self._localize_datetime(self.tz, self.datetime)
        return timesince(localized, target)


def register_all(register, values):
    for value in values:
        if hasattr(value, "tag_name"):
            register.tag(value.tag_name, value.parse)


register = template.Library()
register_all(register, globals().values())



from django import template

register = template.Library()

@register.filter
def human_size(num):
    num = float(num)
    for x in ['bytes','KB','MB','GB','TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0



import sys
import logging
from functools import partial
from optparse import make_option, OptionParser

from dwdj.strutil import dedent

from django.core.management.base import BaseCommand as DjBaseCommand

class BaseCommand(DjBaseCommand):
    """ A less terrible base class for Djagno management commands.

        Changes:

        * Tracebacks are turned on by default
        * Verbosity is controlled by a counter instead of a number (ex,
          ``-vvv`` instead of ``-v3``) and are used to set up logging levels
          too (disable by setting ``logging_enabled = False``).
        * Will override a named logger (see ``logging_handler_name``) with the
          level and formatter settings from the command line (or add a new
          stderr logger if no matching logger is found).
        * Subclassess can set ``option_list`` directly - the base options are
          stored in ``base_option_list`` and merged with ``get_option_list``.

        Example::

            from dwdj.management.base import BaseCommand, make_option

            class MyCommand(BaseCommand):
                option_list = [
                    make_option("-f", "--foo"),
                ]

                def handle(self, *args, **options):
                    pass
    """

    base_option_list = [
        make_option('-v', '--verbose', action="count", dest="verbosity"),
        make_option('-q', '--quiet', action="store_const", const=-1, dest="verbosity"),
        make_option('--verbose-log', action="store_true", help=dedent("""
            Use a more verbose logging format.
        """)),
        make_option('--settings', help=dedent("""
            The Python path to a settings module, e.g.
            "myproject.settings.main". If this isn\'t provided, the
            DJANGO_SETTINGS_MODULE environment variable will be used.')
        """)),
        make_option('--pythonpath', help=dedent("""
            A directory to add to the Python path, e.g.
            "/home/djangoprojects/myproject".
        """)),
        make_option('--traceback', action='store_true', default=True, help=dedent("""
            Show full exception traceback
        """)),
        make_option('--no-traceback', action='store_false', dest="traceback",
                    help=dedent("""
            Hide exception traceback (implied by --quiet)
        """))
    ]

    option_list = []

    logging_enabled = True
    logging_handler_name = "stderr"
    logging_format = '%(levelname)s [%(name)s]: %(message)s'
    logging_format_verbose =  '%(asctime)s %(levelname)s [%(processName)s:%(threadName)s:%(name)s]: %(message)s'

    def get_option_list(self):
        return self.base_option_list + self.option_list

    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``OptionParser`` which will be used to
        parse the arguments to this command.

        """
        parser = OptionParser(prog=prog_name,
                              usage=self.usage(subcommand),
                              version=self.get_version(),
                              option_list=self.get_option_list())
        original_parse = parser.parse_args
        parser.parse_args = partial(self._override_parse_args, parser,
                                    original_parse)
        return parser

    def _override_parse_args(self, parser, original_parse, *argv, **kwargs):
        options, args = original_parse(*argv, **kwargs)
        self.logging_setup(options)
        if options.verbosity < 0:
            options.traceback = False
        return (options, args)

    def logging_setup(self, options):
        verbosity = options.verbosity
        level = (
            logging.CRITICAL if verbosity < 0 else
            logging.ERROR if verbosity == 0 else
            logging.WARNING if verbosity == 1 else
            logging.INFO if verbosity == 2 else
            logging.DEBUG
        )
        format = (
            self.logging_format if not options.verbose_log else
            self.logging_format_verbose
        )
        logger, handler = self.logging_get_handler()
        formatter = logging.Formatter(format)
        handler.setFormatter(formatter)
        logger.setLevel(level)
        handler.setLevel(level)

    def logging_get_handler(self):
        logger = logging.getLogger("")
        for handler in logger.handlers:
            if handler.name == self.logging_handler_name:
                return logger, handler
        handler = logging.StreamHandler(sys.stderr)
        handler.name = self.logging_handler_name
        logger.addHandler(handler)
        return logger, handler



import sys
import os

from django.core.management import setup_environ

did_setup = False

def setup_test_environment():
    """ Setup a Django environment which can be used for testing.
        This will be called from ./__init__.py, and the environment in
        'helper_project/' will be used. """
    global did_setup
    if did_setup:
        return
    did_setup = True

    sys.path.append(os.path.join(os.path.dirname(__file__), "helper_project/"))
    import helper_project.settings
    setup_environ(helper_project.settings)



import json

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client
from nose.tools import assert_equal

def assert_redirects(resp, url, status=(301, 302)):
    assert resp.status_code in status or resp.status_code == status, \
        "response status %r not valid (not %s)" %(resp.status, status)
    assert_equal(resp.get("Location", "<no location header>"),
                 "http://testserver" + url)

def loggedin_client(username="admin"):
    c = Client()
    result = c.login(username=username, password="asdf")
    assert result, "login failed"
    return c

def get_ok(c, *args, **kwargs):
    resp = c.get(*args, **kwargs)
    if resp.status_code != 200:
        raise AssertionError("unexpected status code: %r; headers:\n%s"
                             %(resp.status_code,
                               "\n".join("%s: %s" %x for x in resp.items())))

    return resp


class APITestCase(TestCase):
    def url(self, name, *args, **kwargs):
        return reverse(name, args=args, kwargs=kwargs, current_app="api")

    def login(self, username="admin", password="asdf"):
        login_result = self.client.login(username=username,
                                         password=password)
        assert login_result, "login failed"

    def call(self, method, url, *args, **kwargs):
        return getattr(self.client, method)("/api/" + url, {
            "__kwargs": json.dumps(kwargs),
            "__args": json.dumps(args),
        })

    def resp_data(self, resp):
        try:
            resp_obj = json.loads(resp.content)
        except ValueError:
            print "error decoding json:"
            print resp.content
            raise
        assert resp_obj["ok"], "response returned error: %r" %(resp_obj, )
        print resp_obj
        return resp_obj["data"]



# NOTE: This function has been copied almost verbatim from
#       django/contrib/admin/__init__.py
# A flag to tell us if autodiscover is running.  autodiscover will set this to
# True while running, and False when it finishes.

LOADING = False
def discover_providers(module):
    """
    Auto-discover INSTALLED_APPS modules which provide 'module' and fail
    silently when not present. This forces an import on them to register any
    bits they may want.

    Note: a list of discovered modules is intentionally not returned -- things
    should not be "magically" detected, rather they should have to explicitly
    publish themselves (eg, see admin.py).
    """
    # Bail out if discover didn't finish loading from a previous call so that
    # we avoid running discover again when the URLConf is loaded by the
    # exception handler to resolve the handler500 view.  This prevents a module
    # with errors from re-registering models and raising a spurious
    # AlreadyRegistered exception (see #8245).
    global LOADING
    if LOADING:
        return
    LOADING = True

    import imp
    from django.conf import settings

    for app in settings.INSTALLED_APPS:
        # For each app, we need to look for "module" inside that app's
        # package. We can't use os.path here -- recall that modules may be
        # imported different ways (think zip files) -- so we need to get
        # the app's __path__ and look for "module" on that path.

        # Step 1: find out the app's __path__.  Import errors here will (and
        # should) bubble up, but a missing __path__ (which is legal, but weird)
        # fails silently -- apps that do weird things with __path__ might
        # not work so well...
        try:
            app_path = __import__(app, {}, {}, [app.split('.')[-1]]).__path__
        except AttributeError:
            continue

        # Step 2: use imp.find_module to find the app's "module". For some
        # reason imp.find_module raises ImportError if the app can't be found
        # but doesn't actually try to import the module. So skip this app if
        # its "module" doesn't exist
        try:
            imp.find_module(module, app_path)
        except ImportError:
            continue

        # Step 3: import the app's admin file. If this has errors we want them
        # to bubble up.
        __import__("%s.%s" %(app, module))

    # autodiscover was successful, reset loading flag.
    LOADING = False



from django.conf.urls.defaults import include

def include_factory(package, includefunc=include):
    """ Returns a Django-urls-``include`` function which will resolve names relative
        to ``package``::

            >>> __package__
            'mysite.urls'
            >>> include = include_factory(__package__)
            >>> include('.foo')
            ('mysite.urls.foo', ...)
            >>> include('..foo')
            ('mysite.foo', ...)

        For example::

            include = include_factory(__package__)
            urlpatterns = patterns('',
                url('^wiki/', include('.wiki.urls')),
                url('^accounts/', include('.accounts.urls')),
            )
        """
    pkg_split = package.split(".")
    def include_factory_helper(module, **kwargs):
        if isinstance(module, basestring) and module.startswith("."):
            module_rel = module.lstrip(".")
            num_above = len(module) - len(module_rel) - 1
            module_abs = pkg_split[:len(pkg_split)-num_above] + [module_rel]
            module = ".".join(module_abs)
        return includefunc(module, **kwargs)
    return include_factory_helper
